@@ -35,6 +35,7 @@
 #include "rt-utils.h"
 #include "rt-numa.h"
 #include "rt-error.h"
+#include "histogram.h"
 
 #include <bionic.h>
 
@@ -133,16 +134,13 @@ struct thread_stat {
 	double avg;
 	long *values;
 	long *smis;
-	long *hist_array;
-	long *outliers;
+	struct histogram *hist;
 	pthread_t thread;
 	int threadstarted;
 	int tid;
 	long reduce;
 	long redmax;
 	long cycleofmax;
-	long hist_overflow;
-	long num_outliers;
 	unsigned long smi_count;
 };
 
@@ -216,6 +214,7 @@ static char jsonfile[MAX_PATH];
 
 static struct thread_param **parameters;
 static struct thread_stat **statistics;
+static struct histoset hset;
 
 static void print_stat(FILE *fp, struct thread_param *par, int index, int verbose, int quiet);
 static void rstat_print_stat(struct thread_param *par, int index, int verbose, int quiet);
@@ -405,7 +404,14 @@ static int get_smi_counter(int fd, unsigned long *counter)
 
 #include <cpuid.h>
 
-/* Based on turbostat's check */
+/* Guess if the CPU has an SMI counter in the model-specific registers (MSR).
+ *
+ * This is true for Intel x86 CPUs after Nehalem (2008). However, it's not
+ * possible to detect the feature directly, (or at least, turbostat.c doesn't
+ * know how to do it either) so we assume it's true for all x86 CPUs.
+ * Previously, this function had an explicit allowlist, which required updates
+ * every time a new CPU generation was released.
+ */
 static int has_smi_counter(void)
 {
 	unsigned int ebx, ecx, edx, max_level;
@@ -429,88 +435,6 @@ static int has_smi_counter(void)
 	if (!(edx & (1 << 5)))
 		return 0;
 
-	model = (((fms >> 16) & 0xf) << 4) + ((fms >> 4) & 0xf);
-
-	/* Based on intel_model_duplicates */
-	switch (model) {
-	case 0x1A:      /* INTEL_FAM6_NEHALEM_EP */
-	case 0x1E:      /* INTEL_FAM6_NEHALEM */
-	case 0x1F:      /* Core i7 and i5 Processor - Nehalem */
-	case 0x25:      /* INTEL_FAM6_WESTMERE */
-	case 0x2C:      /* INTEL_FAM6_WESTMERE_EP */
-		model = 0x1E;      /* INTEL_FAM6_NEHALEM */
-		break;
-	case 0x2E:      /* INTEL_FAM6_NEHALEM_EX */
-	case 0x2F:      /* INTEL_FAM6_WESTMERE_EX */
-		model = 0x2E;      /* INTEL_FAM6_NEHALEM_EX */
-		break;
-	case 0x85:      /* INTEL_FAM6_XEON_PHI_KNM */
-		model = 0x57;      /* INTEL_FAM6_XEON_PHI_KNL */
-		break;
-	case 0x4F:      /* INTEL_FAM6_BROADWELL_X */
-	case 0x56:      /* INTEL_FAM6_BROADWELL_D */
-		model = 0x4F;      /* INTEL_FAM6_BROADWELL_X */
-		break;
-	case 0x4E:      /* INTEL_FAM6_SKYLAKE_L */
-	case 0x5E:      /* INTEL_FAM6_SKYLAKE */
-	case 0x8E:      /* INTEL_FAM6_KABYLAKE_L */
-	case 0x9E:      /* INTEL_FAM6_KABYLAKE */
-	case 0xA6:      /* INTEL_FAM6_COMETLAKE_L */
-	case 0xA5:      /* INTEL_FAM6_COMETLAKE */
-		model = 0x4E;      /* INTEL_FAM6_SKYLAKE_L */
-		break;
-	case 0x7E:      /* INTEL_FAM6_ICELAKE_L */
-	case 0x9D:      /* INTEL_FAM6_ICELAKE_NNPI */
-	case 0x8C:      /* INTEL_FAM6_TIGERLAKE_L */
-	case 0x8D:      /* INTEL_FAM6_TIGERLAKE */
-	case 0xA7:      /* INTEL_FAM6_ROCKETLAKE */
-	case 0x8A:      /* INTEL_FAM6_LAKEFIELD */
-	case 0x97:      /* INTEL_FAM6_ALDERLAKE */
-	case 0x9A:      /* INTEL_FAM6_ALDERLAKE_L */
-		model = 0x66;      /* INTEL_FAM6_CANNONLAKE_L */
-		break;
-	case 0x9C:      /* INTEL_FAM6_ATOM_TREMONT_L */
-		model = 0x96;      /* INTEL_FAM6_ATOM_TREMONT */
-		break;
-	case 0x6C:      /* INTEL_FAM6_ICELAKE_D */
-	case 0x8F:      /* INTEL_FAM6_SAPPHIRERAPIDS_X */
-		model = 0x6A;      /* INTEL_FAM6_ICELAKE_X */
-		break;
-	}
-
-	/* Based on probe_nhm_msrs */
-	switch (model) {
-	case 0x1E:      /* INTEL_FAM6_NEHALEM */
-	case 0x2E:      /* INTEL_FAM6_NEHALEM_EX */
-	case 0x2A:      /* INTEL_FAM6_SANDYBRIDGE */
-	case 0x2D:      /* INTEL_FAM6_SANDYBRIDGE_X */
-	case 0x3A:      /* INTEL_FAM6_IVYBRIDGE */
-	case 0x3E:      /* INTEL_FAM6_IVYBRIDGE_X */
-	case 0x3C:      /* INTEL_FAM6_HASWELL */
-	case 0x46:      /* INTEL_FAM6_HASWELL_G */
-	case 0x3F:      /* INTEL_FAM6_HASWELL_X */
-	case 0x45:      /* INTEL_FAM6_HASWELL_L */
-	case 0x3D:      /* INTEL_FAM6_BROADWELL */
-	case 0x47:      /* INTEL_FAM6_BROADWELL_G */
-	case 0x4F:      /* INTEL_FAM6_BROADWELL_X */
-	case 0x4E:      /* INTEL_FAM6_SKYLAKE_L */
-	case 0x66:      /* INTEL_FAM6_CANNONLAKE_L */
-	case 0x55:      /* INTEL_FAM6_SKYLAKE_X */
-	case 0x6A:      /* INTEL_FAM6_ICELAKE_X */
-	case 0x37:      /* INTEL_FAM6_ATOM_SILVERMONT */
-	case 0x4D:      /* INTEL_FAM6_ATOM_SILVERMONT_D */
-	case 0x4C:      /* INTEL_FAM6_ATOM_AIRMONT */
-	case 0x57:      /* INTEL_FAM6_XEON_PHI_KNL */
-	case 0x5C:      /* INTEL_FAM6_ATOM_GOLDMONT */
-	case 0x7A:      /* INTEL_FAM6_ATOM_GOLDMONT_PLUS */
-	case 0x5F:      /* INTEL_FAM6_ATOM_GOLDMONT_D */
-	case 0x96:      /* INTEL_FAM6_ATOM_TREMONT */
-	case 0x86:      /* INTEL_FAM6_ATOM_TREMONT_D */
-		break;
-	default:
-		return 0;
-	}
-
 	return 1;
 }
 #else
@@ -777,15 +701,8 @@ static void *timerthread(void *param)
 		}
 
 		/* Update the histogram */
-		if (histogram) {
-			if (diff >= histogram) {
-				stat->hist_overflow++;
-				if (stat->num_outliers < histogram)
-					stat->outliers[stat->num_outliers++] = stat->cycles;
-			} else {
-				stat->hist_array[diff]++;
-			}
-		}
+		if (histogram)
+			hist_sample(stat->hist, diff);
 
 		stat->cycles++;
 
@@ -1407,12 +1324,9 @@ static void print_tids(struct thread_param *par[], int nthreads)
 static void print_hist(struct thread_param *par[], int nthreads)
 {
 	int i, j;
-	unsigned long long int log_entries[nthreads+1];
 	unsigned long maxmax, alloverflows;
 	FILE *fd;
 
-	bzero(log_entries, sizeof(log_entries));
-
 	if (use_histfile) {
 		fd = fopen(histfile, "w");
 		if (!fd) {
@@ -1425,30 +1339,15 @@ static void print_hist(struct thread_param *par[], int nthreads)
 
 	fprintf(fd, "# Histogram\n");
 	for (i = 0; i < histogram; i++) {
-		unsigned long long int allthreads = 0;
+		unsigned long flags = 0;
+		char buf[64];
 
-		fprintf(fd, "%06d ", i);
+		snprintf(buf, sizeof(buf), "%06d ", i);
 
-		for (j = 0; j < nthreads; j++) {
-			unsigned long curr_latency=par[j]->stats->hist_array[i];
-			fprintf(fd, "%06lu", curr_latency);
-			if (j < nthreads - 1)
-				fprintf(fd, "\t");
-			log_entries[j] += curr_latency;
-			allthreads += curr_latency;
-		}
-		if (histofall && nthreads > 1) {
-			fprintf(fd, "\t%06llu", allthreads);
-			log_entries[nthreads] += allthreads;
-		}
-		fprintf(fd, "\n");
+		if (histofall)
+			flags |= HSET_PRINT_SUM;
+		hset_print_bucket(&hset, fd, buf, i, flags);
 	}
-	fprintf(fd, "# Total:");
-	for (j = 0; j < nthreads; j++)
-		fprintf(fd, " %09llu", log_entries[j]);
-	if (histofall && nthreads > 1)
-		fprintf(fd, " %09llu", log_entries[nthreads]);
-	fprintf(fd, "\n");
 	fprintf(fd, "# Min Latencies:");
 	for (j = 0; j < nthreads; j++)
 		fprintf(fd, " %05lu", par[j]->stats->min);
@@ -1471,8 +1370,8 @@ static void print_hist(struct thread_param *par[], int nthreads)
 	fprintf(fd, "# Histogram Overflows:");
 	alloverflows = 0;
 	for (j = 0; j < nthreads; j++) {
-		fprintf(fd, " %05lu", par[j]->stats->hist_overflow);
-		alloverflows += par[j]->stats->hist_overflow;
+		fprintf(fd, " %05lu", par[j]->stats->hist->oflow_count);
+		alloverflows += par[j]->stats->hist->oflow_count;
 	}
 	if (histofall && nthreads > 1)
 		fprintf(fd, " %05lu", alloverflows);
@@ -1480,11 +1379,8 @@ static void print_hist(struct thread_param *par[], int nthreads)
 
 	fprintf(fd, "# Histogram Overflow at cycle number:\n");
 	for (i = 0; i < nthreads; i++) {
-		fprintf(fd, "# Thread %d:", i);
-		for (j = 0; j < par[i]->stats->num_outliers; j++)
-			fprintf(fd, " %05lu", par[i]->stats->outliers[j]);
-		if (par[i]->stats->num_outliers < par[i]->stats->hist_overflow)
-			fprintf(fd, " # %05lu others", par[i]->stats->hist_overflow - par[i]->stats->num_outliers);
+		fprintf(fd, "# Thread %d: ", i);
+		hist_print_oflows(par[i]->stats->hist, fd);
 		fprintf(fd, "\n");
 	}
 	if (smi) {
@@ -1800,28 +1696,20 @@ rstat_err:
 static void write_stats(FILE *f, void *data __attribute__ ((unused)))
 {
 	struct thread_param **par = parameters;
-	int i, j;
-	unsigned comma;
+	int i;
 	struct thread_stat *s;
 
 	fprintf(f, "  \"num_threads\": %d,\n", num_threads);
 	fprintf(f, "  \"resolution_in_ns\": %u,\n", use_nsecs);
 	fprintf(f, "  \"thread\": {\n");
 	for (i = 0; i < num_threads; i++) {
-		fprintf(f, "    \"%u\": {\n", i);
-
-		fprintf(f, "      \"histogram\": {");
 		s = par[i]->stats;
-		for (j = 0, comma = 0; j < histogram; j++) {
-			if (s->hist_array[j] == 0)
-				continue;
-			fprintf(f, "%s", comma ? ",\n" : "\n");
-			fprintf(f, "        \"%u\": %ld", j, s->hist_array[j]);
-			comma = 1;
+		fprintf(f, "    \"%u\": {\n", i);
+		if (s->hist) {
+			fprintf(f, "      \"histogram\": {");
+			hist_print_json(s->hist, f);
+			fprintf(f, "      },\n");
 		}
-		if (comma)
-			fprintf(f, "\n");
-		fprintf(f, "      },\n");
 		fprintf(f, "      \"cycles\": %ld,\n", s->cycles);
 		fprintf(f, "      \"min\": %ld,\n", s->min);
 		fprintf(f, "      \"max\": %ld,\n", s->max);
@@ -2003,6 +1891,10 @@ int main(int argc, char **argv)
 	/* Set-up shm */
 	rstat_setup();
 
+	if (histogram && hset_init(&hset, num_threads, 1, histogram, histogram))
+		fatal("failed to allocate histogram of size %d for %d threads\n",
+		      histogram, num_threads);
+
 	parameters = calloc(num_threads, sizeof(struct thread_param *));
 	if (!parameters)
 		goto out;
@@ -2078,18 +1970,8 @@ int main(int argc, char **argv)
 			fatal("error allocating thread status struct for thread %d\n", i);
 		memset(stat, 0, sizeof(struct thread_stat));
 
-		/* allocate the histogram if requested */
-		if (histogram) {
-			int bufsize = histogram * sizeof(long);
-
-			stat->hist_array = threadalloc(bufsize, node);
-			stat->outliers = threadalloc(bufsize, node);
-			if (stat->hist_array == NULL || stat->outliers == NULL)
-				fatal("failed to allocate histogram of size %d on node %d\n",
-				      histogram, i);
-			memset(stat->hist_array, 0, bufsize);
-			memset(stat->outliers, 0, bufsize);
-		}
+		if (histogram)
+			stat->hist = &hset.histos[i];
 
 		if (verbose) {
 			int bufsize = VALBUF_SIZE * sizeof(long);
@@ -2227,13 +2109,8 @@ int main(int argc, char **argv)
 	if (trigger)
 		trigger_print();
 
-	if (histogram) {
+	if (histogram)
 		print_hist(parameters, num_threads);
-		for (i = 0; i < num_threads; i++) {
-			threadfree(statistics[i]->hist_array, histogram*sizeof(long), parameters[i]->node);
-			threadfree(statistics[i]->outliers, histogram*sizeof(long), parameters[i]->node);
-		}
-	}
 
 	if (tracelimit) {
 		print_tids(parameters, num_threads);
@@ -2275,5 +2152,6 @@ int main(int argc, char **argv)
 	if (rstat_fd >= 0)
 		shm_unlink(shm_name);
 
+	hset_destroy(&hset);
 	exit(ret);
 }
